@@ -3,18 +3,19 @@
 
 import os
 import errno
-import inspect
 
 import shutil
 import tarfile
 import tempfile
 
-import re
+import datetime
 
 from . import jsonfiles, string
 from .errors import *
+from .fcollection import FCollection
+from .folder import Folder
 from .simulation import Simulation
-from . import checkers
+from . import checkers as default_checkers
 
 class Manager():
 	'''
@@ -22,18 +23,77 @@ class Manager():
 
 	Parameters
 	----------
-	folder : Folder
-		The folder to manage.
+	folder : Folder|string
+		The folder to manage. Either a `Folder` instance or the path to the folder (used to create a `Folder` instance).
+
+	readonly : bool
+		If `True`, open an instance in "read only" mode. Running indicator file is ignored, but only extraction is allowed.
+
+	Raises
+	------
+	ManagerAlreadyRunningError
+		A Manager instance is already running.
 	'''
 
-	def __init__(self, folder):
-		self._folder = folder
+	def __init__(self, folder, *, readonly = False):
+		self._folder = folder if type(folder) is Folder else Folder(folder)
 
-		self._simulations_list_file = os.path.join(self._folder.folder, '.simulations.list')
+		self._simulations_list_file = self._folder.confFilePath('simulations.list')
 		self._simulations_list_dict = None
 
-		self._checkers_regex_compiled = None
-		self._checkers_list = None
+		self._checkers = None
+
+		self._readonly = readonly
+
+		self._running_indicator_filename = self._folder.confFilePath('manager.running')
+		self._delete_running_indicator = False
+
+		if not(self._readonly):
+			if os.path.isfile(self._running_indicator_filename):
+				self._delete_running_indicator = False
+				raise ManagerAlreadyRunningError()
+
+			# Add a file into the configuration folder to indicate a Manager instance is currently running
+			with open(self._running_indicator_filename, 'w') as f:
+				f.write(str(datetime.datetime.now()))
+
+			self._delete_running_indicator = True
+
+	def __enter__(self):
+		'''
+		Context manager so we can use `with` instead of manually calling `close()`.
+		'''
+
+		return self
+
+	def __exit__(self, type, value, traceback):
+		'''
+		Ensure `close()` is called when exiting the context manager.
+		'''
+
+		self.close()
+
+	def close(self):
+		'''
+		Delete the "running indicator".
+		Should always be called before destroying the instance to ensure the indicator is deleted.
+		'''
+
+		if self._delete_running_indicator:
+			os.unlink(self._running_indicator_filename)
+
+	@property
+	def folder(self):
+		'''
+		Return the `Folder` instance.
+
+		Returns
+		-------
+		folder : Folder
+			The instance used by the manager.
+		'''
+
+		return self._folder
 
 	@property
 	def _simulations_list(self):
@@ -58,100 +118,38 @@ class Manager():
 	def saveSimulationsList(self):
 		'''
 		Save the list of simulations.
+
+		Raises
+		------
+		OperationNotAllowed
+			The list cannot be written in read only mode.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		jsonfiles.write(self._simulations_list, self._simulations_list_file)
 
 	@property
-	def _checkers_regex(self):
-		'''
-		Regex to determine whether a function's name corresponds to a checker.
-
-		Returns
-		-------
-		regex : re.Pattern
-			The checkers regex.
-		'''
-
-		if self._checkers_regex_compiled is None:
-			self._checkers_regex_compiled = re.compile(r'^(file|folder|global)_([A-Za-z0-9_]+)$')
-
-		return self._checkers_regex_compiled
-
-	@property
-	def _checkers(self):
+	def checkers(self):
 		'''
 		Get the list of available checkers.
 
 		Returns
 		-------
-		checkers : dict
-			Checkers, sorted by category.
+		checkers : FCollection
+			The collection of checkers.
 		'''
 
-		if self._checkers_list is None:
-			self._checkers_list = {cat: {} for cat in ['file', 'folder', 'global']}
-			self.loadCheckersFromModule(checkers)
+		if self._checkers is None:
+			self._checkers = FCollection(
+				categories = ['file', 'folder', 'global'],
+				filter_regex = r'^(?P<category>file|folder|global)_(?P<name>[A-Za-z0-9_]+)$'
+			)
 
-		return self._checkers_list
+			self._checkers.loadFromModule(default_checkers)
 
-	def loadCheckersFromModule(self, module):
-		'''
-		Load all checkers in a given module.
-
-		Parameters
-		----------
-		module : Module
-			Module (already loaded) where are defined the checkers.
-		'''
-
-		for function in inspect.getmembers(module, inspect.isfunction):
-			checker_match = self._checkers_regex.match(function[0])
-
-			if checker_match:
-				self.setChecker(checker_match.group(1), checker_match.group(2), function[1])
-
-	def setChecker(self, category, checker_name, checker):
-		'''
-		Set (add or replace) a checker.
-
-		Parameters
-		----------
-		category : str
-			Category of the checker (`file`, `folder` or `global`).
-
-		checker_name : str
-			Name of checker.
-
-		checker : function
-			Checker to register (callback function).
-		'''
-
-		if not(category in self._checkers):
-			raise CheckersCategoryNotFoundError(category)
-
-		self._checkers[category][checker_name] = checker
-
-	def removeChecker(self, category, checker_name):
-		'''
-		Remove a checker.
-
-		Parameters
-		----------
-		category : str
-			Category of the checker (`file`, `folder` or `global`).
-
-		checker_name : str
-			Name of checker.
-		'''
-
-		if not(category in self._checkers):
-			raise CheckersCategoryNotFoundError(category)
-
-		if not(checker_name in self._checkers[category]):
-			raise CheckerNotFoundError(checker_name, category)
-
-		del self._checkers[category][checker_name]
+		return self._checkers
 
 	def checkIntegrity(self, simulation):
 		'''
@@ -181,19 +179,27 @@ class Manager():
 
 					if 'checks' in output:
 						for checker_name in output['checks']:
-							if not(checker_name in self._checkers[checkers_cat]):
+							try:
+								checker = self.checkers.get(checker_name, category = checkers_cat)
+
+							except FCollectionFunctionNotFoundError:
 								raise CheckerNotFoundError(checker_name, checkers_cat)
 
-							if not(self._checkers[checkers_cat][checker_name](simulation, parsed_name)):
-								return False
+							else:
+								if not(checker(simulation, parsed_name)):
+									return False
 
 		if 'checks' in self._folder.settings['output']:
 			for checker_name in self._folder.settings['output']['checks']:
-				if not(checker_name in self._checkers['global']):
+				try:
+					checker = self.checkers.get(checker_name, category = 'global')
+
+				except FCollectionFunctionNotFoundError:
 					raise CheckerNotFoundError(checker_name, 'global')
 
-				if not(self._checkers['global'][checker_name](simulation, tree)):
-					return False
+				else:
+					if not(checker(simulation, tree)):
+						return False
 
 		return True
 
@@ -248,14 +254,14 @@ class Manager():
 
 		return len(self._simulations_list)
 
-	def settingsOf(self, simulation_name):
+	def settingsOf(self, simulation_id):
 		'''
 		Return the whole settings set of a simulation.
 
 		Parameters
 		----------
-		simulation_name : str
-			Name of the simulation.
+		simulation_id : str
+			Identifier of the simulation, either its name or its hashed settings.
 
 		Raises
 		------
@@ -269,10 +275,17 @@ class Manager():
 		'''
 
 		try:
-			return string.toObject(self._simulations_list[simulation_name]['settings'])
+			if len(simulation_id) == 32:
+				settings_str = [infos['settings'] for infos in self._simulations_list.values() if infos['name'] == simulation_id][0]
 
-		except KeyError:
-			raise SimulationNotFoundError(simulation_name)
+			else:
+				settings_str = self._simulations_list[simulation_id]['settings']
+
+		except (IndexError, KeyError):
+			raise SimulationNotFoundError(simulation_id)
+
+		else:
+			return string.toObject(settings_str)
 
 	def add(self, simulation, save_list = True):
 		'''
@@ -288,6 +301,9 @@ class Manager():
 
 		Raises
 		------
+		OperationNotAllowed
+			Addition is not allowed in read only mode.
+
 		SimulationFolderNotFoundError
 			The folder indicated in the simulation does not exist.
 
@@ -295,12 +311,15 @@ class Manager():
 			At least one integrity check failed.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		simulation = Simulation.ensureType(simulation, self._folder)
 
 		if not(os.path.isdir(simulation['folder'])):
 			raise SimulationFolderNotFoundError(simulation['folder'])
 
-		settings_str = string.fromObject(simulation.settings_dict)
+		settings_str = string.fromObject(simulation.settings)
 		settings_hashed = string.hash(settings_str)
 		simulation_name = string.uniqueID()
 
@@ -331,12 +350,18 @@ class Manager():
 
 		Raises
 		------
+		OperationNotAllowed
+			Deletion is not allowed in read only mode.
+
 		SimulationNotFoundError
 			The simulation does not exist in the list.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		simulation = Simulation.ensureType(simulation, self._folder)
-		settings_hashed = string.hash(string.fromObject(simulation.settings_dict))
+		settings_hashed = string.hash(string.fromObject(simulation.settings))
 
 		if not(settings_hashed in self._simulations_list):
 			raise SimulationNotFoundError(settings_hashed)
@@ -371,7 +396,7 @@ class Manager():
 		'''
 
 		simulation = Simulation.ensureType(simulation, self._folder)
-		settings_hashed = string.hash(string.fromObject(simulation.settings_dict))
+		settings_hashed = string.hash(string.fromObject(simulation.settings))
 
 		if not(settings_hashed in self._simulations_list):
 			raise SimulationNotFoundError(settings_hashed)
@@ -388,7 +413,7 @@ class Manager():
 		self.uncompress(simulation_name, simulation['folder'])
 
 		if settings_file:
-			jsonfiles.write(simulation.settings_dict, os.path.join(simulation['folder'], settings_file))
+			simulation.writeSettingsFile(settings_file)
 
 	def batchAction(self, simulations, action, args = {}, *, save_list = True, errors_store = (), errors_pass = (Error), callback = None):
 		'''
@@ -523,8 +548,16 @@ class Manager():
 		----------
 		callback : function
 			Function to call at each treated simulation.
+
+		Raises
+		------
+		OperationNotAllowed
+			Checking the folder is not allowed in read only mode.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		test_infos = self._simulations_list[list(self._simulations_list.keys())[0]]
 
 		if not(type(test_infos) is dict) or not('name' in test_infos and 'settings' in test_infos):
@@ -557,8 +590,16 @@ class Manager():
 		----------
 		callback : function
 			Function to call at each treated simulation.
+
+		Raises
+		------
+		OperationNotAllowed
+			Updating is not allowed in read only mode.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		new_simulations_list = {}
 
 		for settings_hashed, infos in self._simulations_list.items():
@@ -567,7 +608,7 @@ class Manager():
 				'settings': string.toObject(infos['settings'])
 			}, self._folder)
 
-			settings_str = string.fromObject(simulation.settings_dict)
+			settings_str = string.fromObject(simulation.settings)
 			settings_hashed = string.hash(settings_str)
 
 			new_simulations_list[settings_hashed] = {
@@ -593,15 +634,24 @@ class Manager():
 				simulation : Simulation
 					A Simulation object representing the settings of the current simulation.
 
-			Returned value is ignored.
+			The function returns the new settings of the simulation.
+			If `None`, the settings are not changed.
 
 		simulations_settings : list
 			List of simulations to transform (only their settings). If `None`, will transform all the stored simulations.
 
 		callback : function
 			Function to call once a simulation has been transformed.
+
+		Raises
+		------
+		OperationNotAllowed
+			Transformation is not allowed in read only mode.
 		'''
 
+		if self._readonly:
+			raise OperationNotAllowed()
+
 		if not(simulations_settings):
 			simulations_settings = [string.toObject(infos['settings']) for infos in self._simulations_list.values()]
 
@@ -613,13 +663,58 @@ class Manager():
 				'settings': settings
 			}, self._folder)
 
-			settings_hashed = string.hash(string.fromObject(simulation.settings_dict))
+			settings_hashed = string.hash(string.fromObject(simulation.settings))
 			simulation_name = self._simulations_list[settings_hashed]['name']
 
 			self.uncompress(simulation_name, simulation_dir)
-			transformation(simulation)
+			new_settings = transformation(simulation)
 			os.unlink(os.path.join(self._folder.folder, f'{simulation_name}.tar.bz2'))
 			self.compress(simulation_dir, simulation_name)
 
+			if not(new_settings is None):
+				new_simulation = Simulation.ensureType({
+					'folder': simulation_dir,
+					'settings': new_settings
+				}, self._folder)
+
+				new_settings_str = string.fromObject(new_simulation.settings)
+				new_settings_hashed = string.hash(new_settings_str)
+
+				del self._simulations_list[settings_hashed]
+
+				self._simulations_list[new_settings_hashed] = {
+					'name': simulation_name,
+					'settings': new_settings_str
+				}
+
+				self.saveSimulationsList()
+
 			if not(callback is None):
 				callback()
+
+	def clear(self, callback = None):
+		'''
+		Delete all simulations in the folder.
+
+		Parameters
+		----------
+		callback : function
+			Function to call at each deleted simulation.
+
+		Raises
+		------
+		OperationNotAllowed
+			Clearing is not allowed in read only mode.
+		'''
+
+		if self._readonly:
+			raise OperationNotAllowed()
+
+		for infos in self._simulations_list.values():
+			os.unlink(os.path.join(self._folder.folder, f'{infos["name"]}.tar.bz2'))
+
+			if not(callback is None):
+				callback()
+
+		self._simulations_list_dict = {}
+		self.saveSimulationsList()
