@@ -30,6 +30,21 @@ class RemoteFolder():
 		self._configuration = folder_conf
 		self._local = (self._configuration['host'] == 'local')
 
+	def __enter__(self):
+		'''
+		Context manager to call `open()` and `close()` automatically.
+		'''
+
+		self.open()
+		return self
+
+	def __exit__(self, type, value, traceback):
+		'''
+		Ensure `close()` is called when exiting the context manager.
+		'''
+
+		self.close()
+
 	def open(self):
 		'''
 		Open the connection.
@@ -42,7 +57,16 @@ class RemoteFolder():
 			self._ssh = paramiko.SSHClient()
 			self._ssh.load_system_host_keys()
 			self._ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
-			self._ssh.connect(self._configuration['host'], username = self._configuration['user'])
+
+			connect_params = {'username': self._configuration['user']}
+
+			try:
+				connect_params['port'] = self._configuration['port']
+
+			except KeyError:
+				pass
+
+			self._ssh.connect(self._configuration['host'], **connect_params)
 
 			self._sftp = self._ssh.open_sftp()
 
@@ -201,7 +225,7 @@ class RemoteFolder():
 
 		return filename
 
-	def getFileContents(self, remote_path, *, callback = None):
+	def getFileContents(self, remote_path):
 		'''
 		Retrieve the content of a remote file.
 
@@ -210,26 +234,46 @@ class RemoteFolder():
 		remote_path : str
 			Path of the remote file to read.
 
-		callback : function
-			Function to transform each line.
-
 		Returns
 		-------
-		content : list
-			Content of the file, as a list of lines.
+		content : str
+			Content of the file, as a string.
 		'''
 
-		content = []
-
 		with self._sftp.open(remote_path, 'r') as f:
-			if callback is None:
-				content = f.readlines()
+			return f.read()
 
-			else:
-				for line in f:
-					content.append(callback(line))
+	def putFileContents(self, remote_path, content):
+		'''
+		Write the content of a remote file.
 
-		return content
+		Parameters
+		----------
+		remote_path : str
+			Path of the remote file to write.
+
+		content : str
+			Content to write.
+		'''
+
+		with self._sftp.open(remote_path, 'w') as f:
+			f.write(content)
+
+	def appendToFile(self, remote_path, content):
+		'''
+		Append a string to a remote file.
+
+		Parameters
+		----------
+		remote_path : str
+			Path of the remote file to write into.
+
+		content : str
+			Content to append.
+		'''
+
+		with self._sftp.open(remote_path, 'a') as f:
+			f.write(content)
 
 	def makedirs(self, directory):
 		'''
